@@ -23,7 +23,6 @@ import sampledb
 import sampledb.config
 import sampledb.logic
 import tests.conftest
-import tests.test_utils
 
 
 def scroll_to(driver, x, y):
@@ -375,7 +374,7 @@ temp_dir = tempfile.mkdtemp()
 try:
     os.mkdir(os.path.join(temp_dir, 'uploaded_files'))
     sampledb.config.FILE_STORAGE_PATH = os.path.join(temp_dir, 'uploaded_files')
-    app = tests.test_utils.create_app()
+    app = tests.conftest.create_app()
     with app.app_context():
         user = sampledb.models.User(
             name="Example User",
@@ -432,7 +431,7 @@ try:
             'propertyOrder': ['name', 'tags', 'hazards']
         }
         instrument_action = sampledb.logic.actions.create_action(
-            action_type=sampledb.logic.actions.ActionType.SAMPLE_CREATION,
+            action_type_id=sampledb.models.ActionType.SAMPLE_CREATION,
             name="Sample Creation",
             description="This is an example action",
             schema=schema,
@@ -465,7 +464,7 @@ try:
         # disable Chrome sandbox for root in GitLab CI
         if 'CI' in os.environ and getpass.getuser() == 'root':
             options.add_argument('--no-sandbox')
-        with contextlib.contextmanager(tests.test_utils.create_flask_server)(app) as flask_server:
+        with contextlib.contextmanager(tests.conftest.create_flask_server)(app) as flask_server:
             with contextlib.closing(Chrome(options=options)) as driver:
                 time.sleep(5)
                 object_permissions(flask_server.base_url, driver)
