@@ -6,6 +6,7 @@ RESTful API for iffSamples
 from flask_restful import Api
 from .objects import Object, Objects, ObjectVersion, ObjectVersions
 from .actions import Action, Actions
+from .action_types import ActionType, ActionTypes
 from .files import ObjectFile, ObjectFiles
 from .instruments import Instrument, Instruments
 from .instrument_log import InstrumentLogEntry, InstrumentLogEntries, InstrumentLogEntryFileAttachment, InstrumentLogEntryFileAttachments, InstrumentLogEntryObjectAttachment, InstrumentLogEntryObjectAttachments, InstrumentLogCategory, InstrumentLogCategories
@@ -20,6 +21,8 @@ api.add_resource(ObjectVersions, '/api/v1/objects/<int:object_id>/versions/', en
 api.add_resource(ObjectVersion, '/api/v1/objects/<int:object_id>/versions/<int:version_id>', endpoint='api.object_version')
 api.add_resource(Actions, '/api/v1/actions/', endpoint='api.actions')
 api.add_resource(Action, '/api/v1/actions/<int:action_id>', endpoint='api.action')
+api.add_resource(ActionTypes, '/api/v1/action_types/', endpoint='api.action_types')
+api.add_resource(ActionType, '/api/v1/action_types/<int(signed=True):type_id>', endpoint='api.action_type')
 api.add_resource(Instruments, '/api/v1/instruments/', endpoint='api.instruments')
 api.add_resource(Instrument, '/api/v1/instruments/<int:instrument_id>', endpoint='api.instrument')
 api.add_resource(InstrumentLogEntries, '/api/v1/instruments/<int:instrument_id>/log_entries/', endpoint='api.instrument_log_entries')
