@@ -18,10 +18,11 @@ import werkzeug.utils
 
 from . import frontend
 from .. import logic
+from .. import models
 from ..logic import user_log, object_log, comments, object_sorting
-from ..logic.actions import ActionType, get_action
+from ..logic.actions import get_action, get_actions, get_action_type, get_action_types
 from ..logic.action_permissions import get_user_action_permissions
-from ..logic.object_permissions import Permissions, get_user_object_permissions, object_is_public, get_object_permissions_for_users, set_object_public, set_user_object_permissions, set_group_object_permissions, set_project_object_permissions, get_objects_with_permissions, get_object_permissions_for_groups, get_object_permissions_for_projects, request_object_permissions
+from ..logic.object_permissions import Permissions, get_user_object_permissions, object_is_public, get_object_permissions_for_users, set_object_public, set_user_object_permissions, set_group_object_permissions, set_project_object_permissions, get_objects_with_permissions, get_object_info_with_permissions, get_object_permissions_for_groups, get_object_permissions_for_projects, request_object_permissions
 from ..logic.datatypes import JSONEncoder
 from ..logic.users import get_user, get_users, get_users_by_name
 from ..logic.schemas import validate, generate_placeholder
@@ -33,7 +34,7 @@ from ..logic.object_log import ObjectLogEntryType
 from ..logic.projects import get_project, get_user_projects, get_user_project_permissions
 from ..logic.locations import get_location, get_object_ids_at_location, get_object_location_assignment, get_object_location_assignments, get_locations, assign_location_to_object, get_locations_tree
 from ..logic.files import FileLogEntryType
-from ..logic.errors import GroupDoesNotExistError, ObjectDoesNotExistError, UserDoesNotExistError, ActionDoesNotExistError, ValidationError, ProjectDoesNotExistError, LocationDoesNotExistError
+from ..logic.errors import GroupDoesNotExistError, ObjectDoesNotExistError, UserDoesNotExistError, ActionDoesNotExistError, ValidationError, ProjectDoesNotExistError, LocationDoesNotExistError, ActionTypeDoesNotExistError
 from .objects_forms import ObjectPermissionsForm, ObjectForm, ObjectVersionRestoreForm, ObjectUserPermissionsForm, CommentForm, ObjectGroupPermissionsForm, ObjectProjectPermissionsForm, FileForm, FileInformationForm, FileHidingForm, ObjectLocationAssignmentForm, ExternalLinkForm, ObjectPublicationForm, CopyPermissionsForm
 from ..utils import object_permissions_required
 from .utils import jinja_filter, generate_qrcode
@@ -136,12 +137,24 @@ def objects():
             action_type = action.type
         else:
             action = None
-            action_type = flask.request.args.get('t', '')
-            action_type = {
-                'samples': ActionType.SAMPLE_CREATION,
-                'measurements': ActionType.MEASUREMENT,
-                'simulations': ActionType.SIMULATION
-            }.get(action_type, None)
+            action_type_id = flask.request.args.get('t', '')
+            if action_type_id is not None:
+                try:
+                    action_type_id = int(action_type_id)
+                except ValueError:
+                    # ensure old links still function
+                    action_type_id = {
+                        'samples': models.ActionType.SAMPLE_CREATION,
+                        'measurements': models.ActionType.MEASUREMENT,
+                        'simulations': models.ActionType.SIMULATION
+                    }.get(action_type_id, None)
+            if action_type_id is not None:
+                try:
+                    action_type = get_action_type(action_type_id)
+                except ActionTypeDoesNotExistError:
+                    action_type = None
+            else:
+                action_type = None
         try:
             project_id = int(flask.request.args.get('project', ''))
         except ValueError:
@@ -288,7 +301,7 @@ def objects():
                 limit=limit,
                 offset=offset,
                 action_id=action_id,
-                action_type=action_type,
+                action_type_id=action_type.id if action_type is not None else None,
                 project_id=project_id,
                 object_ids=object_ids,
                 num_objects_found=num_objects_found_list
@@ -323,8 +336,6 @@ def objects():
     # TODO: select display_properties? nested display_properties? find common properties? use searched for properties?
     display_properties = []
     display_property_titles = {}
-    sample_ids = set()
-    measurement_ids = set()
     if action is not None:
         action_schema = action.schema
         display_properties = action_schema.get('displayProperties', [])
@@ -337,19 +348,6 @@ def objects():
                 obj['display_properties'][property_name] = None
                 continue
             obj['display_properties'][property_name] = (obj['data'][property_name], obj['schema']['properties'][property_name])
-            if obj['schema']['properties'][property_name]['type'] == 'sample':
-                sample_ids.add(obj['data'][property_name]['object_id'])
-            elif obj['schema']['properties'][property_name]['type'] == 'measurement':
-                measurement_ids.add(obj['data'][property_name]['object_id'])
-
-    samples = {
-        sample_id: get_object(object_id=sample_id)
-        for sample_id in sample_ids
-    }
-    measurements = {
-        measurement_id: get_object(object_id=measurement_id)
-        for measurement_id in measurement_ids
-    }
     if action_id is None:
         show_action = True
     else:
@@ -370,7 +368,6 @@ def objects():
         action=action,
         action_id=action_id,
         action_type=action_type,
-        ActionType=ActionType,
         project=project,
         project_id=project_id,
         location_id=location_id,
@@ -378,8 +375,7 @@ def objects():
         user_id=user_id,
         user=user,
         doi=doi,
-        samples=samples,
-        measurements=measurements,
+        get_object_if_current_user_has_read_permissions=get_object_if_current_user_has_read_permissions,
         build_modified_url=build_modified_url,
         sorting_property=sorting_property_name,
         sorting_order=sorting_order_name,
@@ -435,7 +431,7 @@ def apply_action_to_form_data(action, form_data):
         parent_id_prefix = action_id_prefix
         new_form_data = {}
         for name in form_data:
-            if not name.startswith(parent_id_prefix):
+            if not name.startswith(parent_id_prefix + '__'):
                 new_form_data[name] = form_data[name]
             else:
                 item_index, id_suffix = name[len(parent_id_prefix) + 2:].split('__', 1)
@@ -678,36 +674,36 @@ def show_object_form(object, action, previous_object=None, should_upgrade_schema
         except ValueError:
             flask.abort(400)
 
-    # TODO: make this search more narrow
-    samples = get_objects_with_permissions(
-        user_id=flask_login.current_user.id,
-        permissions=Permissions.READ,
-        action_type=ActionType.SAMPLE_CREATION
-    )
-    measurements = get_objects_with_permissions(
-        user_id=flask_login.current_user.id,
-        permissions=Permissions.READ,
-        action_type=ActionType.MEASUREMENT
-    )
-    if object is not None:
-        samples = [
-            sample
-            for sample in samples
-            if sample.object_id != object.object_id
-        ]
-        measurements = [
-            measurement
-            for measurement in measurements
-            if measurement.object_id != object.object_id
-        ]
+    if not flask.current_app.config["LOAD_OBJECTS_IN_BACKGROUND"]:
+        referencable_objects = get_objects_with_permissions(
+            user_id=flask_login.current_user.id,
+            permissions=Permissions.READ
+        )
+        if object is not None:
+            referencable_objects = [
+                referencable_object
+                for referencable_object in referencable_objects
+                if referencable_object.object_id != object.object_id
+            ]
 
-    existing_objects = get_objects_with_permissions(
-        user_id=flask_login.current_user.id,
-        permissions=Permissions.GRANT
-    )
+    else:
+        referencable_objects = []
+        existing_objects = []
+
+    action_type_id_by_action_id = {}
+    for action_type in get_action_types():
+        for action in get_actions(action_type.id):
+            action_type_id_by_action_id[action.id] = action_type.id
 
     tags = [{'name': tag.name, 'uses': tag.uses} for tag in logic.tags.get_tags()]
+    users = get_users(exclude_hidden=True)
     if object is None:
+        if not flask.current_app.config["LOAD_OBJECTS_IN_BACKGROUND"]:
+            existing_objects = get_objects_with_permissions(
+                user_id=flask_login.current_user.id,
+                permissions=Permissions.GRANT
+            )
+
         return flask.render_template(
             'objects/forms/form_create.html',
             action_id=action_id,
@@ -718,12 +714,14 @@ def show_object_form(object, action, previous_object=None, should_upgrade_schema
             form_data=form_data,
             previous_actions=serializer.dumps(previous_actions),
             form=form,
-            samples=samples,
-            measurements=measurements,
             can_copy_permissions=True,
             existing_objects=existing_objects,
+            referencable_objects=referencable_objects,
+            action_type_id_by_action_id=action_type_id_by_action_id,
+            ActionType=models.ActionType,
             datetime=datetime,
             tags=tags,
+            users=users,
             may_create_log_entry=may_create_log_entry,
             instrument_log_categories=instrument_log_categories,
             create_log_entry_default=create_log_entry_default,
@@ -741,14 +739,27 @@ def show_object_form(object, action, previous_object=None, should_upgrade_schema
             form_data=form_data,
             previous_actions=serializer.dumps(previous_actions),
             form=form,
-            samples=samples,
-            measurements=measurements,
+            referencable_objects=referencable_objects,
+            action_type_id_by_action_id=action_type_id_by_action_id,
+            ActionType=models.ActionType,
             datetime=datetime,
             tags=tags,
+            users=users,
             mode=mode
         )
 
 
+def get_object_if_current_user_has_read_permissions(object_id):
+    user_id = flask_login.current_user.id
+    try:
+        permissions = get_user_object_permissions(object_id, user_id)
+    except ObjectDoesNotExistError:
+        return None
+    if Permissions.READ not in permissions:
+        return None
+    return get_object(object_id)
+
+
 @frontend.route('/objects/<int:object_id>', methods=['GET', 'POST'])
 @object_permissions_required(Permissions.READ, on_unauthorized=on_unauthorized)
 def object(object_id):
@@ -758,6 +769,7 @@ def object(object_id):
     user_permissions = get_user_object_permissions(object_id=object_id, user_id=flask_login.current_user.id)
     user_may_edit = Permissions.WRITE in user_permissions
     user_may_grant = Permissions.GRANT in user_permissions
+    user_may_use_as_template = Permissions.READ in get_user_action_permissions(object.action_id, user_id=flask_login.current_user.id)
     action = get_action(object.action_id)
     if action.schema != object.schema:
         new_schema_available = True
@@ -768,23 +780,8 @@ def object(object_id):
     if not user_may_edit and flask.request.args.get('mode', '') == 'upgrade':
         return flask.abort(403)
     if flask.request.method == 'GET' and flask.request.args.get('mode', '') not in ('edit', 'upgrade'):
-        # TODO: make this search more narrow
-        samples = get_objects_with_permissions(
-            user_id=flask_login.current_user.id,
-            permissions=Permissions.READ,
-            action_type=ActionType.SAMPLE_CREATION
-        )
-        measurements = get_objects_with_permissions(
-            user_id=flask_login.current_user.id,
-            permissions=Permissions.READ,
-            action_type=ActionType.MEASUREMENT
-        )
         instrument = action.instrument
-        object_type = {
-            ActionType.SAMPLE_CREATION: "Sample",
-            ActionType.MEASUREMENT: "Measurement",
-            ActionType.SIMULATION: "Simulation"
-        }.get(action.type, "Object")
+        object_type = action.type.object_name
         object_log_entries = object_log.get_object_log_entries(object_id=object_id, user_id=flask_login.current_user.id)
 
         if user_may_edit:
@@ -816,7 +813,7 @@ def object(object_id):
             possible_responsible_users.append((str(user.id), '{} (#{})'.format(user.name, user.id)))
         location_form.responsible_user.choices = possible_responsible_users
 
-        measurement_actions = logic.actions.get_actions(logic.actions.ActionType.MEASUREMENT)
+        measurement_actions = logic.actions.get_actions(models.ActionType.MEASUREMENT)
         favorite_action_ids = logic.favorites.get_user_favorite_action_ids(flask_login.current_user.id)
         favorite_measurement_actions = [
             action
@@ -895,8 +892,6 @@ def object(object_id):
             restore_form=None,
             version_id=object.version_id,
             user_may_grant=user_may_grant,
-            samples=samples,
-            measurements=measurements,
             favorite_measurement_actions=favorite_measurement_actions,
             FileLogEntryType=FileLogEntryType,
             file_information_form=FileInformationForm(),
@@ -905,8 +900,10 @@ def object(object_id):
             related_objects_tree=related_objects_tree,
             object_publications=object_publications,
             user_may_link_publication=user_may_link_publication,
+            user_may_use_as_template=user_may_use_as_template,
             publication_form=publication_form,
             get_object=get_object,
+            get_object_if_current_user_has_read_permissions=get_object_if_current_user_has_read_permissions,
             get_object_location_assignment=get_object_location_assignment,
             get_user=get_user,
             get_location=get_location,
@@ -1083,6 +1080,37 @@ def search():
     return flask.render_template('search.html')
 
 
+@frontend.route('/objects/referencable')
+@flask_login.login_required
+def referencable_objects():
+    required_perm = Permissions.READ
+    if 'required_perm' in flask.request.args:
+        try:
+            required_perm = Permissions.from_name(flask.request.args['required_perm'])
+        except ValueError:
+            try:
+                required_perm = Permissions(int(flask.request.args['required_perm']))
+            except ValueError:
+                return {
+                    "message": "argument {} is not a valid permission.".format(flask.request.args['required_perm'])
+                }, 400
+
+    referencable_objects = get_object_info_with_permissions(
+        user_id=flask_login.current_user.id,
+        permissions=required_perm,
+    )
+
+    def dictify(x):
+        return {
+            'id': x.object_id,
+            'text': '{} (#{})'.format(x.name_text, x.object_id),
+            'action_id': x.action_id,
+            'max_permission': x.max_permission
+        }
+
+    return {'referencable_objects': [dictify(x) for x in referencable_objects]}
+
+
 @frontend.route('/objects/<int:object_id>/permissions/request', methods=['POST'])
 @flask_login.login_required
 def object_permissions_request(object_id):
@@ -1377,20 +1405,30 @@ def new_object():
 
     previous_object = None
     action = None
-    if action_id:
-        try:
-            action = get_action(action_id)
-        except ActionDoesNotExistError:
-            return flask.abort(404)
-        if Permissions.READ not in get_user_action_permissions(action_id, user_id=flask_login.current_user.id):
-            return flask.abort(404)
     if previous_object_id:
         try:
             previous_object = get_object(previous_object_id)
         except ObjectDoesNotExistError:
+            flask.flash("This object does not exist.", 'error')
             return flask.abort(404)
         if Permissions.READ not in get_user_object_permissions(user_id=flask_login.current_user.id, object_id=previous_object_id):
+            flask.flash("You do not have the required permissions to use this object as a template.", 'error')
+            return flask.abort(403)
+        if action_id:
+            if action_id != str(previous_object.action_id):
+                flask.flash("This object was created with a different action.", 'error')
+                return flask.abort(400)
+            else:
+                action_id = previous_object.action_id
+    if action_id:
+        try:
+            action = get_action(action_id)
+        except ActionDoesNotExistError:
+            flask.flash("This action does not exist.", 'error')
             return flask.abort(404)
+        if Permissions.READ not in get_user_action_permissions(action_id, user_id=flask_login.current_user.id):
+            flask.flash("You do not have the required permissions to use this action.", 'error')
+            return flask.abort(403)
 
     placeholder_data = {}
 
@@ -1441,25 +1479,22 @@ def object_version(object_id, version_id):
     user_may_grant = Permissions.GRANT in user_permissions
     action = get_action(object.action_id)
     instrument = action.instrument
-    object_type = {
-        ActionType.SAMPLE_CREATION: "Sample",
-        ActionType.MEASUREMENT: "Measurement",
-        ActionType.SIMULATION: "Simulation"
-    }.get(action.type, "Object")
     return flask.render_template(
         'objects/view/base.html',
         is_archived=True,
-        object_type=object_type,
+        object_type=action.type.object_name,
         action=action,
         instrument=instrument,
         schema=object.schema,
         data=object.data,
         last_edit_datetime=object.utc_datetime,
         last_edit_user=get_user(object.user_id),
+        get_object_if_current_user_has_read_permissions=get_object_if_current_user_has_read_permissions,
         object_id=object_id,
         version_id=version_id,
         link_version_specific_rdf=True,
         restore_form=form,
+        get_user=get_user,
         user_may_grant=user_may_grant
     )
 
@@ -1525,15 +1560,20 @@ def object_permissions(object_id):
         projects = [project for project in projects if project.id not in project_permissions]
         add_project_permissions_form = ObjectProjectPermissionsForm()
         copy_permissions_form = CopyPermissionsForm()
-        existing_objects = get_objects_with_permissions(
-            user_id=flask_login.current_user.id,
-            permissions=Permissions.GRANT
-        )
-        copy_permissions_form.object_id.choices = [
-            (str(existing_object.id), existing_object.data['name']['text'])
-            for existing_object in existing_objects
-            if existing_object.id != object_id
-        ]
+        if not flask.current_app.config["LOAD_OBJECTS_IN_BACKGROUND"]:
+            existing_objects = get_objects_with_permissions(
+                user_id=flask_login.current_user.id,
+                permissions=Permissions.GRANT
+            )
+            copy_permissions_form.object_id.choices = [
+                (str(existing_object.id), existing_object.data['name']['text'])
+                for existing_object in existing_objects
+                if existing_object.id != object_id
+            ]
+            if len(copy_permissions_form.object_id.choices) == 0:
+                copy_permissions_form = None
+        else:
+            copy_permissions_form.object_id.choices = []
     else:
         edit_user_permissions_form = None
         add_user_permissions_form = None
@@ -1577,16 +1617,18 @@ def update_object_permissions(object_id):
     add_project_permissions_form = ObjectProjectPermissionsForm()
     copy_permissions_form = CopyPermissionsForm()
     if 'copy_permissions' in flask.request.form:
-
-        existing_objects = get_objects_with_permissions(
-            user_id=flask_login.current_user.id,
-            permissions=Permissions.GRANT
-        )
-        copy_permissions_form.object_id.choices = [
-            (str(existing_object.id), existing_object.data['name']['text'])
-            for existing_object in existing_objects
-            if existing_object.id != object_id
-        ]
+        if not flask.current_app.config["LOAD_OBJECTS_IN_BACKGROUND"]:
+            existing_objects = get_objects_with_permissions(
+                user_id=flask_login.current_user.id,
+                permissions=Permissions.GRANT
+            )
+            copy_permissions_form.object_id.choices = [
+                (str(existing_object.id), existing_object.data['name']['text'])
+                for existing_object in existing_objects
+                if existing_object.id != object_id
+            ]
+        else:
+            copy_permissions_form.object_id.choices = []
         if copy_permissions_form.validate_on_submit():
             logic.object_permissions.copy_permissions(object_id, int(copy_permissions_form.object_id.data))
             logic.object_permissions.set_user_object_permissions(object_id, flask_login.current_user.id, Permissions.GRANT)
