@@ -12,9 +12,6 @@ import sampledb.logic
 import sampledb.models
 
 
-from tests.test_utils import flask_server, app, app_context
-
-
 @pytest.fixture
 def auth_user(flask_server):
     with flask_server.app.app_context():
@@ -47,7 +44,7 @@ def user(auth_user):
 @pytest.fixture
 def action():
     action = sampledb.logic.actions.create_action(
-        action_type=sampledb.logic.actions.ActionType.SAMPLE_CREATION,
+        action_type_id=sampledb.models.ActionType.SAMPLE_CREATION,
         name="",
         description="",
         schema={
