@@ -12,9 +12,6 @@ import sampledb.logic
 import sampledb.models
 
 
-from tests.test_utils import flask_server, app, app_context
-
-
 @pytest.fixture
 def auth_user(flask_server):
     with flask_server.app.app_context():
@@ -47,7 +44,7 @@ def user(auth_user):
 @pytest.fixture
 def action():
     action = sampledb.logic.actions.create_action(
-        action_type=sampledb.logic.actions.ActionType.SAMPLE_CREATION,
+        action_type_id=sampledb.models.ActionType.SAMPLE_CREATION,
         name="",
         description="",
         schema={
@@ -68,7 +65,7 @@ def action():
 @pytest.fixture
 def other_action():
     other_action = sampledb.logic.actions.create_action(
-        action_type=sampledb.logic.actions.ActionType.MEASUREMENT,
+        action_type_id=sampledb.models.ActionType.MEASUREMENT,
         name="",
         description="",
         schema={
@@ -678,6 +675,19 @@ def test_get_objects_by_action_type(flask_server, auth, user, other_user, action
             "data": object.data
         }
     ]
+    r = requests.get(flask_server.base_url + 'api/v1/objects/', auth=auth, allow_redirects=False, params={
+        'action_type_id': sampledb.models.ActionType.SAMPLE_CREATION
+    })
+    assert r.status_code == 200
+    assert r.json() == [
+        {
+            "object_id": object.object_id,
+            "version_id": object.version_id,
+            "action_id": object.action_id,
+            "schema": object.schema,
+            "data": object.data
+        }
+    ]
     r = requests.get(flask_server.base_url + 'api/v1/objects/', auth=auth, allow_redirects=False, params={
         'action_type': 'measurement'
     })
