@@ -5,7 +5,6 @@
 
 import requests
 import pytest
-import bcrypt
 from bs4 import BeautifulSoup
 
 import sampledb
@@ -15,9 +14,6 @@ from sampledb.logic.authentication import add_email_authentication
 from sampledb.logic import object_permissions, groups, projects
 
 
-from tests.test_utils import flask_server, app
-
-
 @pytest.fixture
 def user(flask_server):
     with flask_server.app.app_context():
