@@ -7,8 +7,7 @@ import pytest
 
 from sampledb.logic.schemas import validate_schema
 from sampledb.logic.errors import ValidationError
-
-__author__ = 'Florian Rhiem <f.rhiem@fz-juelich.de>'
+from sampledb.models import ActionType
 
 
 def wrap_into_basic_schema(schema, name='other'):
@@ -1024,6 +1023,80 @@ def test_validate_measurement_schema_with_unknown_property():
         validate_schema(wrap_into_basic_schema(schema))
 
 
+def test_validate_object_reference_schema():
+    schema = {
+        'title': 'Example',
+        'type': 'object_reference',
+        'note': 'Example Note',
+        'action_type_id': ActionType.SAMPLE_CREATION
+    }
+    validate_schema(wrap_into_basic_schema(schema))
+
+
+def test_validate_object_reference_schema_with_invalid_note():
+    schema = {
+        'title': 'Example',
+        'type': 'object_reference',
+        'note': 1
+    }
+    with pytest.raises(ValidationError):
+        validate_schema(wrap_into_basic_schema(schema))
+
+
+def test_validate_object_reference_schema_without_note():
+    schema = {
+        'title': 'Example',
+        'type': 'object_reference'
+    }
+    validate_schema(wrap_into_basic_schema(schema))
+
+
+def test_validate_object_reference_schema_with_action_type_id_none():
+    schema = {
+        'title': 'Example',
+        'type': 'object_reference',
+        'note': 'Example Note',
+        'action_type_id': None
+    }
+    validate_schema(wrap_into_basic_schema(schema))
+
+
+def test_validate_object_reference_schema_with_invalid_action_type_id():
+    schema = {
+        'title': 'Example',
+        'type': 'object_reference',
+        'action_type_id': 'measurement'
+    }
+    with pytest.raises(ValidationError):
+        validate_schema(wrap_into_basic_schema(schema))
+
+
+def test_validate_object_reference_schema_without_action_type_id():
+    schema = {
+        'title': 'Example',
+        'type': 'object_reference'
+    }
+    validate_schema(wrap_into_basic_schema(schema))
+
+
+def test_validate_object_reference_schema_with_missing_title():
+    schema = {
+        'type': 'object_reference'
+    }
+    with pytest.raises(ValidationError):
+        validate_schema(wrap_into_basic_schema(schema))
+
+
+def test_validate_object_reference_schema_with_unknown_property():
+    schema = {
+        'title': 'Example',
+        'type': 'object_reference',
+        'action_id': 0
+    }
+    with pytest.raises(ValidationError):
+        validate_schema(wrap_into_basic_schema(schema))
+
+
 def test_validate_object_schema_with_display_properties():
     schema = {
         'title': 'Example',
@@ -1732,3 +1805,48 @@ def test_notebook_templates_with_invalid_type():
     }
     with pytest.raises(ValidationError):
         validate_schema(schema)
+
+
+def test_validate_user_schema():
+    schema = {
+        'title': 'Example User',
+        'type': 'user',
+        'note': 'Example Note'
+    }
+    validate_schema(wrap_into_basic_schema(schema))
+
+
+def test_validate_user_schema_with_invalid_note():
+    schema = {
+        'title': 'Example User',
+        'type': 'user',
+        'note': 1
+    }
+    with pytest.raises(ValidationError):
+        validate_schema(wrap_into_basic_schema(schema))
+
+
+def test_validate_user_schema_without_note():
+    schema = {
+        'title': 'Example User',
+        'type': 'user'
+    }
+    validate_schema(wrap_into_basic_schema(schema))
+
+
+def test_validate_user_schema_with_missing_title():
+    schema = {
+        'type': 'user'
+    }
+    with pytest.raises(ValidationError):
+        validate_schema(wrap_into_basic_schema(schema))
+
+
+def test_validate_user_schema_with_unknown_property():
+    schema = {
+        'title': 'Example User',
+        'type': 'user',
+        'user_role': 'example'
+    }
+    with pytest.raises(ValidationError):
+        validate_schema(wrap_into_basic_schema(schema))
