@@ -8,9 +8,7 @@ import pytest
 import sampledb
 import sampledb.logic
 from sampledb.logic import action_permissions, groups
-from sampledb.models import User, UserType, Action, ActionType, Instrument, Permissions, UserActionPermissions
-
-from ..test_utils import app_context
+from sampledb.models import User, UserType, Action, Instrument, Permissions, UserActionPermissions
 
 
 __author__ = 'Florian Rhiem <f.rhiem@fz-juelich.de>'
@@ -31,7 +29,7 @@ def users():
 @pytest.fixture
 def independent_action():
     action = Action(
-        action_type=ActionType.SAMPLE_CREATION,
+        action_type_id=sampledb.models.ActionType.SAMPLE_CREATION,
         name='Example Action',
         schema={
             'title': 'Example Action',
@@ -51,7 +49,7 @@ def independent_action():
 @pytest.fixture
 def user_action(users):
     action = Action(
-        action_type=ActionType.SAMPLE_CREATION,
+        action_type_id=sampledb.models.ActionType.SAMPLE_CREATION,
         name='Example Action',
         schema={
             'title': 'Example Action',
@@ -85,7 +83,7 @@ def instrument():
 @pytest.fixture
 def instrument_action(instrument):
     action = Action(
-        action_type=ActionType.SAMPLE_CREATION,
+        action_type_id=sampledb.models.ActionType.SAMPLE_CREATION,
         name='Example Action',
         schema={
             'title': 'Example Action',
