@@ -1,9 +1,6 @@
-import itsdangerous
 import sampledb
 import sampledb.models
 
-from ..test_utils import  app
-
 
 def test_generate_token(app):
 
