@@ -8,7 +8,6 @@ import os
 import pytest
 import sampledb.logic
 import sampledb.__main__ as scripts
-from ..test_utils import app_context
 
 
 @pytest.fixture
@@ -24,7 +23,7 @@ def actions(instrument):
     return [
         sampledb.logic.actions.create_action(
             instrument_id=instrument.id,
-            action_type=sampledb.logic.actions.ActionType.SAMPLE_CREATION,
+            action_type_id=sampledb.models.ActionType.SAMPLE_CREATION,
             name=name,
             description='',
             schema=schema
